# -*- coding: utf-8 -*-
"""
# Created on 05/06/2016
# @author: Emptyset
# @contact: emptyset110@gmail
"""
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Action import Action
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置
import socket
import time
import threading

# 以上是自动生成的 #
class SinaL2TCPAction(Action):
	def __init__(	self
				# ,	name
				# ,	producer_list = [
				# 	{
				# 		"name"	:	"SinaLevel2WS"
				# 	,	"producer_name"	:	"SinaL2TCP.quotation"
				# 	,	"query"	:	['quotation']	#只获取10档行情
				# 	}
				# 	]
				,	addr = "127.0.0.1"
				,	port 	= 9999
				, 	**kwargs
				):
		# 设置进程检查消息队列的间隔
		super().__init__( **kwargs )
		self.addr = addr
		self.port = port
		self.establish_connection()
		self.mutex = threading.Lock()

	def establish_connection(self):
		self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		while True:
			try:
				self.s.connect( ( self.addr, self.port ) )
				break
			except Exception as e:
				self.logger.error( "{}:{}建立socket连接失败, {}\n5秒后重试".format(self.addr,self.port,e) )
				time.sleep(5)
		self.logger.info("建立socket成功")

	# 需要重写的方法
	def handler(self, event):
		if isinstance(event.data, list):
			for data in event.data:
				try:
					self.s.send( event.data.encode(encoding="utf-8") )
				except Exception as e:
					self.logger.error( "{},{}".format(e,e.errno) )
					self.logger.error( "TCP服务端关闭，现在重连" )
					self.s.close()
					self.establish_connection()
					time.sleep(2)
		else:
			try:
				# print( "发送{}".format(event.data) )
				self.s.send( event.data.encode(encoding="utf-8") )
			except Exception as e:
					self.logger.error( "{},{}".format(e,e.errno) )
					self.logger.error( "TCP服务端关闭，现在重连" )
					self.s.close()
					self.establish_connection()
					time.sleep(2)



# -*- coding: utf-8 -*-
import sys
import os
def new_vendor(vendorName = None):
	dirVendor = "vendor/"+vendorName
	if os.path.exists(dirVendor):
		print("[创建失败]：目录 "+dirVendor+"已经存在")
	else:
		os.makedirs( dirVendor )
		# 创建vendorNameVendor.py
		f = open( dirVendor + '/' + vendorName + 'Vendor.py' , 'w' , encoding= 'UTF-8')
		f.write(
		"""# -*- coding: utf-8 -*-
\"\"\"
# Created on 
# @author: 
# @contact: 
\"\"\"
# 以下是自动生成的 #
# --- 导入系统配置
import dHydra.core.util as util
from dHydra.core.Vendor import Vendor
from dHydra.config import connection as CON
from dHydra.config import const as C
# --- 导入自定义配置
from .connection import *
from .const import *
from .config import *
# 以上是自动生成的 #

class {}(Vendor):
	def __init__(self):
		pass
		""".format(vendorName+"Vendor")
			)
		f.close()
		# 创建config,connection,const, __init__.py
		f = open( dirVendor + '/config.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirVendor + '/const.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirVendor + '/connection.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirVendor + '/__init__.py', 'w', encoding= 'UTF-8' )
		f.close()

def new_producer(producerName = None):
	dirProducer = "producer/" + producerName
	if os.path.exists(dirProducer):
		print("[创建失败]：目录 "+dirProducer+"已经存在")
	else:
		os.makedirs( dirProducer )
		# 创建producerNameProducer.py
		f = open( dirProducer + '/' + producerName + 'Producer.py', 'w', encoding= 'UTF-8' )
		if producerName == 'Demo':
			demo = """
		# handler是需要被重写的方法，以下demo每隔0.5秒产生一个数据
		# 并且把数据推送给它的订阅者
		import time
		i = 0
		while ( self._active ):
			event = Event(event_type = 'Demo', data = i)
			for q in self._subscriber:
				q.put(event)
				print("DemoProducer:", event.data)
			i += 1
			time.sleep(0.5)
			"""
		else:
			demo = ""
		f.write(
		"""# -*- coding: utf-8 -*-
\"\"\"
# Created on 
# @author: 
# @contact: 
\"\"\"
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Producer import Producer
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置

# 以上是自动生成的 #

class {}Producer(Producer):
	def __init__(self, name = None, **kwargs):
		super().__init__( name=name, **kwargs )

	def handler(self):{}
		pass
		""".format(producerName, demo)
			)
		f.close()
		# 创建config,connection,const, __init__.py
		f = open( dirProducer + '/config.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirProducer + '/const.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirProducer + '/connection.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirProducer + '/__init__.py', 'w', encoding= 'UTF-8' )
		f.close()

def new_action(actionName = None):
	dirAction = "action/" + actionName
	if os.path.exists(dirAction):
		print("[创建失败]：目录 "+dirAction+"已经存在")
	else:
		os.makedirs( dirAction )
		# 创建actionNameAction.py
		f = open( dirAction + '/' + actionName + 'Action.py', 'w', encoding= 'UTF-8' )
		if actionName == "Demo":
			demo="""
		while not self._queue.empty():
			event = self._queue.get(True)
			print("DemoAction:", event.data)
			# 当收到数字15时，就停止action
			if event.data == 15:
				self._stop()
			"""
		else:
			demo = """
		pass"""
		f.write("""# -*- coding: utf-8 -*-
\"\"\"
# Created on 
# @author: 
# @contact: 
\"\"\"
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Action import Action
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置

# 以上是自动生成的 #
class %sAction(Action):
	def __init__(self, name, **kwargs):
		# 用户自定义自动加载的_producer_list
		self._producer_list = [
			{	
				"name"	:	"Demo"
			,	"producer_name"	:	"Demo.Demo"		#这是在action内部给producer起的自定义名字，可随意。一般最好遵守<actionName.producerName>
			}
		]
		# 设置进程检查消息队列的间隔
		self._interval = 0.5
		super().__init__(name, **kwargs)
		print(self._name,"初始化")

	# 需要重写的方法
	def handler(self):
%s
			""" % (actionName, demo)
			)
		f.close()
		# 创建config,connection,const, __init__.py
		f = open( dirAction + '/config.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirAction + '/const.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirAction + '/connection.py', 'w', encoding= 'UTF-8' )
		f.close()
		f = open( dirAction + '/__init__.py', 'w', encoding= 'UTF-8' )
		f.close()

def init( demo = True ):
	os.makedirs( "data" , exist_ok=True )
	os.makedirs( "producer" , exist_ok=True )
	os.makedirs( "vendor" , exist_ok=True )
	os.makedirs( "action" , exist_ok=True )
	print("目录结构已经生成")
	f = open('config.json', 'w', encoding="UTF-8")
	f.write(
		"""{

}""")
	f.close()
	f = open('test.py', 'w', encoding='UTF-8')
	f.write(
		"""# -*- coding: utf-8 -*-
import logging
from dHydra.app import *

def init_loger():
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(lineno)d - %(levelname)s - %(message)s')
	logger = logging.getLogger()
	logger.setLevel(logging.DEBUG)

	# 屏幕日志打印设置
	console_handler = logging.StreamHandler()
	console_handler.setFormatter(formatter)
	console_handler.setLevel(logging.ERROR)
	logger.addHandler(console_handler)

	if not os.path.exists('log'):
		os.makedirs('log')
	# 打开下面的输出到文件
	file_handler = logging.FileHandler('log/error.log')
	file_handler.setLevel(logging.ERROR)
	file_handler.setFormatter(formatter)
	file_handler2 = logging.FileHandler('log/log.log')
	file_handler2.setLevel(logging.INFO)
	file_handler2.setFormatter(formatter)
	
	logger.setLevel(logging.INFO)
	logger.addHandler(file_handler)
	logger.addHandler(file_handler2)

\"\"\"
初始化日志
\"\"\"
init_loger()
		"""
	)
	f.close()
	if os.path.exists('app.py'):
		print("[创建失败]：app.py已经存在")
	else:
		if demo:
			f = open( 'app.py', 'w', encoding= 'UTF-8' )
			f.write(
				"""# -*- coding: utf-8 -*-

import logging
from dHydra.app import *

def init_loger():
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    # 屏幕日志打印设置
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    console_handler.setLevel(logging.ERROR)
    logger.addHandler(console_handler)

    if not os.path.exists('log'):
        os.makedirs('log')
    # 打开下面的输出到文件
    file_handler = logging.FileHandler('log/error.log')
    file_handler.setLevel(logging.ERROR)
    file_handler.setFormatter(formatter)
    file_handler2 = logging.FileHandler('log/log.log')
    file_handler2.setLevel(logging.INFO)
    file_handler2.setFormatter(formatter)
    
    logger.setLevel(logging.INFO)
    logger.addHandler(file_handler)
    logger.addHandler(file_handler2)

\"\"\"
初始化日志
\"\"\"

init_loger()

\"\"\"
读取数据处理模块(Action)列表
\"\"\"
action_list = ['PrintSinaL2']
\"\"\"
生成Action对象并开启actions
\"\"\"
start_action(action_list)

\"\"\"
嗯……只要这样就可以了
\"\"\"
				"""
				)
			f.close()



# -*- coding: utf-8 -*-
"""
# Created on 2016/04/12
# @author:
# @contact:
"""
# 以下是自动生成的 #
# --- 导入系统配置
import dHydra.core.util as util
from dHydra.core.Vendor import Vendor
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Globals import *
# --- 导入自定义配置
from .connection import *
from .const import *
from .config import *
# 以上是自动生成的 #

import requests
import re
from pandas import DataFrame
import pandas as pd
import asyncio
import functools
from datetime import datetime
import json
import base64,binascii,rsa
import getpass


class SinaVendor(Vendor):
	def __init__(self, username = None, pwd = None):
		super().__init__()
		if (username is None):
			if "sinaUsername" in config.keys():
				self.username = config["sinaUsername"]
			else:
				self.username = input('请输入新浪登录帐号：')
		else:
			self.username=username
		if (pwd is None):
			if "sinaPassword" in config.keys():
				self.pwd = config["sinaPassword"]
			else:
				self.pwd = getpass.getpass("输入登录密码（密码不会显示在屏幕上，输入后按回车确定）:")
		else:
			self.pwd = pwd
		self.rsa_pubkey = '10001'
		self.ip = util.get_client_ip()
		self.session = requests.Session()
		self.quote = None
		self.is_login = False
		self.symbols = self.get_symbols()

	def get_verify_code(self):
		verify_code_response = self.session.get("http://login.sina.com.cn/cgi/pin.php")
		# 保存验证码
		image_path = os.path.join(os.getcwd(), 'vcode.png')
		with open(image_path, 'wb') as f:
			f.write(verify_code_response.content)
		verifyCode = input( "验证码图片保存在{}，\n请输入验证码：".format(image_path) )
		return verifyCode

	def login(self, verify = False):
		self.session.get("http://finance.sina.com.cn/realstock/company/sz300204/l2.shtml")
		su = base64.b64encode(self.username.encode('utf-8'))

		preLogin = self.session.get( URL_PRELOGIN, params = PARAM_PRELOGIN( su ), headers = HEADERS_LOGIN)
		preLogin = json.loads( preLogin.text[len("sinaSSOController.preloginCallBack("):-1] )

		sp = self.get_sp( self.pwd, preLogin["pubkey"], int(preLogin["servertime"]), preLogin["nonce"] )

		if verify:
			verifyCode = self.get_verify_code()
		else:
			verifyCode = ''

		self.loginResponse = self.session.post(
			URL_SSOLOGIN
		,	params = PARAM_LOGIN()
		,	data = DATA_LOGIN(
				su = su
			,	servertime = int( preLogin["servertime"] )
			,	nonce = preLogin["nonce"]
			,	rsakv = preLogin["rsakv"]
			,	sp = sp
			,	door = verifyCode
			)
		,	headers = HEADERS_LOGIN
		)
		if (self.loginResponse.json()["retcode"]=='0'):
			print( "登录成功: %s, uid = %s" % ( self.loginResponse.json()["nick"], self.loginResponse.json()["uid"]) )

			i = 0
			for url in self.loginResponse.json()["crossDomainUrlList"]:
				req = self.session.get( url,headers = HEADERS_CROSSDOMAIN( CROSSDOMAIN_HOST[i] ) )
				i += 1
			return True
		elif (self.loginResponse.json()["retcode"] == '4049'):
			print( self.loginResponse.json() )
			self.is_login = self.login(verify = True)
			if self.is_login:
				return True
			else:
				return False
		else:
			print( self.loginResponse.json() )
			return False

	# RSA2 encoding
	def get_sp(self, passwd, pubkey, servertime, nonce):
		key = rsa.PublicKey(int(pubkey, 16), int('10001', 16))
		message = str(servertime) + '\t' + str(nonce) + '\n' + str(passwd)
		passwd = rsa.encrypt(message.encode('utf-8'), key)
		return binascii.b2a_hex(passwd).decode('ascii')

	def get_deal(self, symbol, stime = None, etime = None):
		while not self.is_login:
			self.is_login = self.login()

		if stime is None:
			stime = "09:25:00"
		if etime is None:
			etime = "15:05:00"
		nextPage = True
		page = 1
		l2 = list()
		while (nextPage):
			data = self.session.get(	URL_L2HIST
						,	params 	= 	PARAM_L2HIST(symbol=symbol, page=page, stime=stime, etime=etime)
						,	headers = 	HEADERS_L2(symbol=symbol)
			)
			data = data.text[90:-2]
			data = dict( json.loads(data) )
			count = int( data["result"]["data"]["count"] )
			if ( data["result"]["data"]["data"] is not None ):
				l2.extend(data["result"]["data"]["data"])
			if ( len(l2) == count ):
				nextPage = False
			page+=1
		l2df = pd.DataFrame( l2 )
		l2df = l2df.convert_objects(convert_dates=False,convert_numeric=True,convert_timedeltas=False)
		if not(len(l2df)==0):
			l2df = l2df.set_index("index").sort_index("index")
		return l2df



	# sz,sh的时间戳不同，强烈建议分开调用
	# 如果开启split = True, 则会返回
	def get_realtime_quotes(self, symbols = None, dataframe = True, loop = None, split = False):
		self.quote = None
		if symbols is None:
			symbols = self.symbols
		if loop is None:
			loop = asyncio.get_event_loop()
		else:
			asyncio.set_event_loop(loop)

		if isinstance(symbols, str):
			symbolList = symbols.split(',')
		else:
			symbolList = list(symbols)
		[ symbols_sz, symbols_sh ] = util.split_symbols(symbolList)
		symbolListSlice = util.slice_list(step = 800, dataList = symbols_sz)
		symbolListSlice.extend( util.slice_list(step = 800, dataList = symbols_sh) )
		loop.run_until_complete( self.get_quote_task(symbolListSlice=symbolListSlice, dataframe = dataframe) )
		if dataframe:
			self.quote = self.quote.drop( 'ms', axis = 1 )
			self.quote.convert_objects(convert_dates=False,convert_numeric=True,convert_timedeltas=False)
		if split:
			quote_sz = self.quote[ self.quote.symbol > 'sz' ]
			quote_sh = self.quote[ self.quote.symbol < 'sz' ]
			return [ quote_sz, quote_sh ]
		return self.quote

	@asyncio.coroutine
	def get_quote_task(self, symbolListSlice, dataframe):
		for symbolList in symbolListSlice:
			quote = yield from self.get_quote_coroutine(symbols = symbolList, dataframe=dataframe)
			if self.quote is None:
				self.quote = quote
			else:
				if dataframe:
					self.quote = self.quote.append(quote, ignore_index=True)
				else:
					self.quote.extend(quote)


	@asyncio.coroutine
	def get_quote_coroutine(self, symbols, dataframe):
		loop = asyncio.get_event_loop()
		if isinstance(symbols, list) or isinstance(symbols, set) or isinstance(symbols, tuple):
			symbolList = list(symbols)
		elif isinstance(symbols, str):
			symbolList = symbols.split(',')
		symbols = util.symbols_to_string(symbols)
		url = URL_QUOTATION(symbols)

		retry = True
		while retry:
			try:
				async_req = loop.run_in_executor(None, functools.partial( self.session.get
				,	URL_QUOTATION(symbols)
				,	timeout = 0.1
				) )
				quote = yield from async_req
				retry = False
			except:
				pass
		quote = quote.text
		quote = re.findall(r'\"(.*)\"', quote)

		for i in range( 0, len(quote) ):
			quote[i] = quote[i].split(',')

		if dataframe:
			quote = DataFrame( quote, columns = SINA_QUOTE_COLUMNS )
			quote["symbol"] = symbolList
			quote["time"] = datetime.strptime( quote.iloc[0]["date"] + " " + quote.iloc[0]["time"] , '%Y-%m-%d %H:%M:%S')
		return quote

	# 新浪获取当前全部股票的接口
	# 如果 dataframe = True， 则返回panda.DataFrame格式，
	# 否则 返回list格式，每个list中是一个dict
	def get_today_all(self, node = 'hs_a', dataframe = True):
		import json
		import time

		# start = time.time()
		retry = True
		while retry:
			try:
				response = self.session.get(
					URL_API_MARKET_CENTER_GETHQNODEDATA(node)
				,	headers = HEADERS_GET_TODAY_ALL
				,	timeout = 3
				).text
				retry = False
			except:
				print("获取数据超时，正在重试")
		# 因为返回的json不标准，需要给key加上引号
		response = response.replace(
			"symbol","\"symbol\""
			).replace(
			"code","\"code\""
			).replace(
			"name","\"name\""
			).replace(
			"trade","\"trade\""
			).replace(
			"pricechange","\"pricechange\""
			).replace(
			"pricepercent","\"pricepercent\""
			).replace(
			"buy","\"buy\""
			).replace(
			"sell","\"sell\""
			).replace(
			"settlement","\"settlement\""
			).replace(
			"open","\"open\""
			).replace(
			"high","\"high\""
			).replace(
			"low","\"low\""
			).replace(
			"volume","\"volume\""
			).replace(
			"amount","\"amount\""
			).replace(
			"ticktime","\"ticktime\""
			).replace(
			"per","\"per\""
			).replace(
			"pb","\"pb\""
			).replace(
			"mktcap","\"mktcap\""
			).replace(
			"nmc","\"nmc\""
			).replace(
			"turnoverratio","\"turnoverratio\""
			).replace("change\"per\"cent","\"changepercent\"")
		# print(time.time()-start)
		todayAll = json.loads( response )
		if dataframe:
			todayAll = DataFrame(todayAll)
		return todayAll

	"""
	用于获取当日股票列表
	返回： list
	参数：
		（可选）stockTypeList:
			类型：list
			默认：["hs_a","hs_b"] 代表获取全部沪深AB代码
	"""
	def get_symbols(self, stockTypeList = ["hs_a","hs_b"]):
		symbolList = list()
		for node in stockTypeList:
			symbols = list(self.get_today_all(node = node)["symbol"])
			symbolList.extend(symbols)
		return symbolList

	def get_quote(self, symbols, dataframe = True):
		if isinstance(symbols, list) or isinstance(symbols, set) or isinstance(symbols, tuple):
			symbolList = list(symbols)
		elif isinstance(symbols, str):
			symbolList = symbols.split(',')
		symbols = util.symbols_to_string(symbols)
		url = URL_QUOTATION(symbols)
		retry = True
		while retry:
			try:
				quote  =self.session.get(
						URL_QUOTATION(symbols)
					,	timeout = 0.1
					).text
				retry = False
			except:
				pass
		quoteList = re.findall(r'\"(.*)\"', quote)
		if dataframe:
			for i in range( 0, len(quoteList) ):
				quoteList[i] = quoteList[i].split(',')
		else:
			for i in range( 0, len(quoteList) ):
				quoteList[i] = quoteList[i].split(',')
				quoteList[i].append( symbolList[i] )

		if dataframe:
			df_quote = DataFrame( quoteList, columns = SINA_QUOTE_COLUMNS )
			df_quote = df_quote.drop( 'ms', axis = 1 )
			df_quote["symbol"] = symbolList
			return df_quote
		else:
			return quoteList

	def get_ticktime(self):
		quote = self.get_quote(symbols= ["sz000001","sh600000"])
		szTime = datetime.strptime( quote.iloc[0]["date"] + " " + quote.iloc[0]["time"] , '%Y-%m-%d %H:%M:%S')
		shTime = datetime.strptime( quote.iloc[1]["date"] + " " + quote.iloc[1]["time"] , '%Y-%m-%d %H:%M:%S')
		return [szTime, shTime]



# -*- coding: utf-8 -*-
import threading
import logging
import time

class Thread(threading.Thread):
    def __init__(self, target, name = None, args = list(), kwargs = dict(), cancel_thread = None, on_finished = None, del_thread = None, manager = None):
        super().__init__(target = target, args = args, kwargs = kwargs, name = name)
        self.logger = self.get_logger()
        self.on_finished = on_finished
        self.__target = target
        self.__cancel = False
        if cancel_thread is not None:
            self.__cancel_thread = cancel_thread
        else:
            self.__cancel_thread = self.cancel_thread
        self.__del_thread = del_thread
        self._manager = manager

    def get_logger(self):
        logger = logging.getLogger(self.__class__.__name__)
        return logger

    def cancel_thread(self):
        return False

    def del_thread(self):
        self.logger.info( "Deleting: {}".format( threading.current_thread().name ) )
        return self.__del_thread( name = threading.current_thread().name )

    def run(self):
        while self.__cancel is False:
            result = self.__target()
            if self.__cancel_thread is not None:
                self.__cancel = ( self.__cancel_thread() ) and ( self._manager.threads_num >= self._manager.num_min )

        if self.on_finished is not None:
            self.on_finished( result )
        self.del_thread()

class Manager():
    """
    manager = Manager(   target = some_func
                    ,   args = some_args
                    ,   kwargs = some_kwargs
                    ,   num_start = 10
                    ,   num_max = 100
                    ,   num_min = 2
                    )
    """
    def __init__(   self
                ,   target
                ,   args = list()
                ,   kwargs = dict()
                ,   num_start = 10
                ,   num_max = 100
                ,   num_min = 3
                ,   set_daemon = True
                ,   need_new_thread = None
                ,   cancel_thread = None
                ,   on_finished = None
                ):
        # Private Variables
        self.__target = target
        self.__args = args
        self.__kwargs = kwargs
        self.__num_start = num_start
        self.num_max = num_max
        self.num_min = num_min
        self.__set_daemon = set_daemon
        self.__threads = dict()
        self.__need_new_thread = need_new_thread  # need_new_thread是一个返回布尔值的函数
        self.__cancel_thread = cancel_thread
        self.__on_finished = on_finished
        self.logger = self.get_logger()

        # Initializing Threads
        self.is_running = False
        for i in range( 0, self.__num_start ):
            t = Thread( target = self.__target
                    ,   args = self.__args
                    ,   kwargs = self.__kwargs
                    ,   cancel_thread = self.__cancel_thread
                    ,   on_finished = self.__on_finished
                    ,   del_thread = self.del_thread
                    ,   manager = self
                    )
            t.setDaemon( self.__set_daemon )
            self.__threads[ t.name ] = t

        # 开启manager，用来动态平衡线程数量
        t = threading.Thread( target = self.manager )
        t.setDaemon(True)
        t.start()
        return

    @property
    def threads_num(self):
        return len(self.__threads)

    def get_logger(self):
        logger = logging.getLogger(self.__class__.__name__)
        return logger

    # 开启一个独立的线程，来动态平衡线程数量
    def manager(self):
        while True:
            num_threads = len( self.__threads )
            if self.__need_new_thread():
                if num_threads >= self.__num_max:
                    self.logger.warning( "需要增加线程，但是已达到用户设置的线程数量上限" )
                else:
                    self.new_thread()
                    self.logger.info( "增加了一个线程" )
            time.sleep(3)

    def del_thread(self, name):
        self.logger.info( "{}".format( self.__threads ) )
        if name in self.__threads.keys():
            self.logger.info( "删除线程: {}".format( name ) )
            del self.__threads[ name ]
            return True
        else:
            return False

    def new_thread(self, ):
        if len(self.__threads) < self.__num_max:
            t = Thread( target = self.__target
                    ,   args = self.__args
                    ,   kwargs = self.__kwargs
                    ,   cancel_thread = self.__cancel_thread
                    ,   on_finished = self.__on_finished
                    ,   del_thread = self.del_thread
                    ,   manager = self
                    )
            self.__threads[ t.name ] = t
            t.setDaemon( self.__set_daemon )
            t.start()
            return True
        else:
            return False

    def start(self,):
        self.check_status()
        self.logger.info("start线程")
        for k in self.__threads.keys():
            thread = self.__threads[ k ]
            thread.start()
        self.is_running = True
        self.logger.info("线程全部开启完毕")

        self.check_status()
        return True

    def check_status(self,):
        key_list = list( self.__threads.keys() )
        for k in key_list:
            thread = self.__threads[ k ]
            self.logger.debug("thread.name ={}, thread.ident ={}, thread.is_alive = {}"\
            .format( thread.name, thread.ident, thread.is_alive() ) )



# -*- coding: utf-8 -*-
import logging
from dHydra.app import *

def init_loger():
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(lineno)d - %(levelname)s - %(message)s')
	logger = logging.getLogger()
	logger.setLevel(logging.DEBUG)

	# 屏幕日志打印设置
	console_handler = logging.StreamHandler()
	console_handler.setFormatter(formatter)
	console_handler.setLevel(logging.ERROR)
	logger.addHandler(console_handler)

	if not os.path.exists('log'):
		os.makedirs('log')
	# 打开下面的输出到文件
	file_handler = logging.FileHandler('log/error.log')
	file_handler.setLevel(logging.ERROR)
	file_handler.setFormatter(formatter)
	file_handler2 = logging.FileHandler('log/debug.log')
	file_handler2.setLevel(logging.DEBUG)
	file_handler2.setFormatter(formatter)

	logger.setLevel(logging.INFO)
	logger.addHandler(file_handler)
	logger.addHandler(file_handler2)

"""
初始化日志
"""
init_loger()



# -*- coding: utf-8 -*-
"""
Config
Created on 02/22/2016
@description:	Used for 
@author: 		Wen Gu
@contact: 		emptyset110@gmail.com
"""
VERSION = '0.2.0'

#	路径配置		
#	Path Configuration
PATH_DATA_ROOT = './data/'
PATH_DATA_REALTIME = 'stock_realtime/'
PATH_DATA_L2 = 'stock_l2/'

#	Mongodb连接配置
#	Mongodb Configuration


#	股票指数与代码的转换表（无需修改）
INDEX_LABELS = ['sh', 'sz', 'hs300', 'sz50', 'cyb', 'zxb', 'zx300', 'zh500']
INDEX_LIST = {'sh': 'sh000001', 'sz': 'sz399001', 'hs300': 'sz399300',
              'sz50': 'sh000016', 'zxb': 'sz399005', 'cyb': 'sz399006', 'zx300': 'sz399008', 'zh500':'sh000905',
              'HSCEI': 'sz110010',
              'HSI' : 'sz110000'
              }


# -*- coding: utf-8 -*-
"""
全局变量在这里定义
"""
import traceback
import sys
import os
import json
# print("加载: Globals.py")

# producer
PRODUCER_HASH = dict()
PRODUCER_NAME = dict()

action_dict = dict()


# 读取配置
try:
	f_config = open( os.getcwd() + "/config.json" )
	config = json.load( f_config )
except Exception as e:
	print( "未能正确加载{}，请检查路径，json文档格式，或者\n可以忽略此警告（当不采用config.json来配置账号时）".format( os.getcwd() + "/config.json" ) )


# -*- coding: utf-8 -*-
from dHydra.core.Action import Action
from dHydra.core import util
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
from dHydra.core.Functions import *
from dHydra.core.ThreadManager import Manager
import re
from datetime import datetime
import threading

class PrintSinaL2Action(Action):
	def __init__(self, **kwargs):
		# 用户自定义自动加载的_producer_list
		# if "producer_list" in kwargs.keys():
		# 	self._producer_list = kwargs["producer_list"]
		# else:
		# 	self._producer_list = [
		# 		{
		# 			"name"	:	"SinaLevel2WS"
		# 		,	"producer_name"	:	"PrintSinaL2.SinaLevel2-quotation"
		# 		,	"query"	:	['quotation']	#只获取10档行情
		# 		}
		# 	# ,	{
		# 	# 		"name"	:	"SinaLevel2WS"
		# 	# 	,	"producer_name"	:	"PrintSinaL2.SinaLevel2-deal"
		# 	# 	,	"query"	:	['deal']
		# 	# 	}
		# 	]

		if "raw" in kwargs.keys():
			self.raw = kwargs["raw"]
		else:
			self.raw = True
		# 设置进程检查消息队列的间隔
		super().__init__( **kwargs )
		self.logger.info(self._name +"初始化")
		self.count = 0

	# 需要重写的方法
	def handler(self, event):
		dt = datetime.now()
		if not self.raw:
			event.data = util.ws_parse(message = event.data)
		if isinstance(event.data, list):
			for data in event.data:
				print("PrintSinaL2:{}\n{}".format( dt,data ) )
		else:
			print("PrintSinaL2:{}\n{}".format( dt,event.data ) )

		# self.logger.info("线程退出")



# -*- coding: utf-8 -*-
"""
工具类
Created on 03/01/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import dHydra.config.const as C
import requests
import threading
import asyncio
import math
import time
from datetime import datetime
import pandas
import os
import ntplib
from pandas import DataFrame
from pymongo import MongoClient
import re
import random

def _code_to_symbol(code):
	"""
		生成symbol代码标志
		@author: Jimmy Liu
		@group : waditu
		@contact: jimmysoa@sina.cn
		@modified: Wen Gu
	"""
	if code in C.INDEX_LIST.keys():
		return C.INDEX_LIST[code]
	else:
		if len(code) != 6 :
			return ''
		else:
			return 'sh%s'%code if code[:1] in ['5', '6', '9', '0' ] else 'sz%s'%code

def symbol_list_to_code(symbolList):
	codeList = []
	for symbol in symbolList:
		codeList.append(symbol[2:8])
	return codeList

def code_list_to_symbol(codeList):
	symbolList = []
	for code in codeList:
		symbolList.append( _code_to_symbol(code) )
	return symbolList

def _get_public_ip():
	return requests.get('http://ipinfo.io/ip').text.strip()

def get_client_ip():
	response = requests.get( 'https://ff.sinajs.cn/?_=%s&list=sys_clientip' % int(time.time()*1000) ).text
	ip = re.findall(r'\"(.*)\"', response)
	return ip[0]

# 用于将一个loop交给一个线程来完成一些任务
def thread_loop(loop, tasks):
	# loop = asyncio.new_event_loop()
	asyncio.set_event_loop(loop)
	loop.run_until_complete( asyncio.wait(tasks) )
	loop.close()

# 用于将一个list按一定步长切片，返回这个list切分后的list
def slice_list(step = None, num = None, dataList=None):
	if not ( (step is None) & (num is None) ):
		if num is not None:
			step = math.ceil(len(dataList)/num)
		return [dataList[ i : i + step] for i in range(0, len(dataList), step)]
	else:
		print("step和num不能同时为空")
		return False

# n个任务交给m个Thread完成
def threads_for_tasks(taskList):
	threads = []
	for task in taskList:
		t = threading.Thread(target = task.target, args = task.args)
		threads.append(t)
	for t in threads:
		t.start()
		print("开启线程：",t.name)
	for t in threads:
		t.join()

def symbols_to_string(symbols):
	if isinstance(symbols, list) or isinstance(symbols, set) or isinstance(symbols, tuple) or isinstance(symbols, pandas.Series):
		return ','.join(symbols)
	else:
		return symbols

"""
与时间相关的转化函数
"""
def datetime_to_timestamp(dt, timeFormat = 'ms'):
	if timeFormat == 'ms':
		return int( time.mktime( dt.timetuple() )*1000 )
	elif timeFormat == 's':
		return int( time.mktime( dt.timetuple() ) )

def date_to_timestamp(date, dateFormat = '%Y-%m-%d', timeFormat = 'ms'):
	return datetime_to_timestamp( dt = datetime.strptime(date, dateFormat) , timeFormat = timeFormat)

def string_to_date(date):
	return datetime.strptime(date, "%Y-%m-%d").date()

def timestamp_to_datetime(timestamp, timeFormat='ms'):
	if timeFormat == 'ms':
		timestamp = timestamp/1000
	return datetime.strftime(timestamp)

def time_now():
	return int(time.time()*1000)

# 从国家授时中心获取时间戳
def get_network_time():
	start = time.time()
	c = ntplib.NTPClient()
	response = c.request('pool.ntp.org')
	ts = response.tx_time
	return ts

def check_time(precision = 0.1):
	duration = 2.0
	while duration > precision:
		try:
			print("{}, 开始获取网络时间戳".format( time.time() ))
			start = time.time()
			networkTime = get_network_time()
			end = time.time()
			duration = end-start
		except Exception as e:
			print("获取网络时间出了点小状况，正重试", duration)
	# print("网络耗时：{}".format( duration ) )
	# print("{}, 网络时间戳".format( networkTime ) )
	# print("{}, 现在时间戳".format( time.time()) )
	difference = networkTime - (start+duration)
	print("difference = {}, (本地时间戳+difference)=网络时间戳".format(difference))
	return difference

"""
symbols相关函数
"""
def split_symbols(symbols):
	df = DataFrame(symbols, columns = ['s'])
	sz = list(df[ df.s > 'sz' ]["s"])
	sh = list(df[ df.s < 'sz' ]["s"])
	return [ sz, sh ]

def upper(dataList):
	for i in range( 0, len(dataList) ):
		dataList[i] = dataList[i].upper()
	return dataList

"""
用于解析Sina l2的函数
"""
def ws_parse( message, to_dict = False ):
	dataList = re.findall(r'(?:((?:2cn_)?((?:sh|sz)[\d]{6})(?:_0|_1|_orders|_i)?)(?:=)(.*)(?:\n))',message)
	result = list()
	for data in dataList:
		if (len(data[0])==12):	# quotation
			wstype = 'quotation'
		elif ( (data[0][-2:]=='_0') | (data[0][-2:]=='_1') ):
			wstype = 'deal'
		elif ( data[0][-6:]=='orders' ):
			wstype = 'orders'
		elif ( (data[0][-2:]=='_i') ):
			wstype = 'info'
		else:
			wstype = 'other'
		result = ws_parse_to_list( wstype=wstype,symbol=data[1],data=data[2],result=result, to_dict = to_dict )
	return result

def ws_parse_to_list( wstype, symbol, data, result ,to_dict ):
	data = data.split(',')
	if wstype is 'deal':
		for d in data:
			x = list()
			x.append(wstype)
			x.append(symbol)
			x.extend( d.split('|') )
			if to_dict is True:
				result.append( deal_to_dict( x ) )
			else:
				result.append(x)
	else:
		x = list()
		x.append(wstype)
		x.append(symbol)
		x.extend(data)
		if to_dict is True:
			if wstype is 'quotation':
				result.append( quotation_to_dict( x ) )
			elif wstype is 'info':
				result.append( info_to_dict( x ) )
			elif wstype is 'orders':
				result.append( orders_to_dict(x) )
		else:
			result.append(x)
	return result

def orders_to_dict( orders ):
	"""
	return
	------
	"""
	return orders

def quotation_to_dict( data ):
	"""
	整个转换大约耗时1*10^(-4)s, 其中datetime.strptime()占用较多耗时
	return
	------
	"""
	print("{}, length = {}".format( data, len(data) ))
	quotation = {
		"data_type" : 'quotation'
	,	"symbol": data[1] # "股票代码"
	,   "name": data[2] # "中文名"
	,   "time": datetime.strptime(data[3]+' '+data[4], "%H:%M:%S %Y-%m-%d") # "datetime格式的日期时间"
	,   "last_close": float( data[5] ) # "昨收"
	,   "open": float( data[5] ) # "今开"
	,   "high": float( data[7] ) #"最高价"
	,   "low": float( data[8] ) #"最低价"
	,   "now": float( data[9] ) #"现价"
	,   "status": data[10] #"状态, PH=盘后，PZ=盘中，TP=停牌, WX=午休, LT=临时停牌,KJ=开盘集合竞价,PZ=连续竞价"
	,   "deal_count": float( data[11] ) #"成交笔数"
	,   "total": int( data[12] ) #"成交总量"
	,   "amount": float( data[13] ) #"总成交金额"
	,   "current_ask": int( data[14] ) if data[14] else 0 # "当前委买总量"
	,   "average_ask": float( data[15] ) if data[15] else 0.0 # "加权平均委买价格"
	,   "current_bid": int( data[16] ) if data[16] else 0 # "当前委卖总量"
	,   "average_bid": float( data[17] ) if data[17] else 0.0 # "加权平均委卖价格"
	# ,   "cancel_ask_num" : int( data[18] ) # "买入撤单笔数" ?
	# ,   "cancel_ask_volumn" : int( data[19] ) #"买入撤单量"?
	# ,   "cancel_ask_amount" : float( data[20] ) #"买入撤单总金额"?
	# ,   "cancel_bid" : int( data[21] ) # "卖出撤单笔数" ?
	# ,   "cancel_bid_volumn" : int( data[22] ) # "卖出撤单量"?
	# ,   "cancel_bid_amount" : float( data[23] ) #"卖出撤单总金额"?
	,   "total_ask" : int( data[24] ) if data[24] else 0 # "委买总笔数"
	,   "total_bid" : int( data[25] ) if data[25] else 0 # "委卖总笔数"
	# ,   "ask" : data[26] #"买档位"
	# ,   "bid" : data[27] #"卖档位"
	,   "b1_price": float( data[28] ) if data[28] else 0.0 # 买1价
	,   "b2_price": float( data[29] ) if data[29] else 0.0 # 买2价
	,	"b3_price": float( data[30] ) if data[30] else 0.0 # 买3价
	,	"b4_price": float( data[31] ) if data[31] else 0.0# 买4价
	,	"b5_price": float( data[32] ) if data[32] else 0.0 # 买5价
	,	"b6_price": float( data[33] ) if data[33] else 0.0 # 买6价
	,	"b7_price": float( data[34] ) if data[34] else 0.0 # 买7价
	,	"b8_price": float( data[35] ) if data[35] else 0.0# 买8价
	,	"b9_price": float( data[36] ) if data[36] else 0.0 # 买9价
	,   "b10_price": float( data[37] ) if data[37] else 0.0 # 买10价
	,   "b1_volume": int( data[38] ) if data[38] else 0 # 买1量
	,   "b2_volume": int( data[39] ) if data[39] else 0 # 买2量
	,	"b3_volume": int( data[40] ) if data[40] else 0 # 买3量
	,	"b4_volume": int( data[41] ) if data[41] else 0 # 买4量
	,	"b5_volume": int( data[42] ) if data[42] else 0 # 买5量
	,	"b6_volume": int( data[43] ) if data[43] else 0 # 买6量
	,	"b7_volume": int( data[44] ) if data[44] else 0 # 买7量
	,	"b8_volume": int( data[45] ) if data[45] else 0 # 买8量
	,	"b9_volume": int( data[46] ) if data[46] else 0 # 买9量
	,   "b10_volume": int( data[47] ) if data[47] else 0 # 买10量
	,   "a1_price": float( data[48] ) if data[48] else 0.0 # 卖1价
	,   "a2_price": float( data[49] ) if data[49] else 0.0 # 卖2价
	,	"a3_price": float( data[50] ) if data[50] else 0.0 # 卖3价
	,	"a4_price": float( data[51] ) if data[51] else 0.0 # 卖4价
	,	"a5_price": float( data[52] ) if data[52] else 0.0 # 卖5价
	,	"a6_price": float( data[53] ) if data[53] else 0.0 # 卖6价
	,	"a7_price": float( data[54] ) if data[54] else 0.0 # 卖7价
	,	"a8_price": float( data[55] ) if data[55] else 0.0 # 卖8价
	,	"a9_price": float( data[56] ) if data[56] else 0.0 # 卖9价
	,   "a10_price": float( data[57] ) if data[57] else 0.0 # 卖10价
	,   "a1_volume": int( data[58] ) if data[58] else 0 #卖1量
	,   "a2_volume": int( data[59] ) if data[59] else 0 # 卖2量
	,	"a3_volume": int( data[60] ) if data[60] else 0 # 卖3量
	,	"a4_volume": int( data[61] ) if data[61] else 0 # 卖4量
	,	"a5_volume": int( data[62] ) if data[62] else 0 # 卖5量
	,	"a6_volume": int( data[63] ) if data[63] else 0 # 卖6量
	,	"a7_volume": int( data[64] ) if data[64] else 0 # 卖7量
	,	"a8_volume": int( data[65] ) if data[65] else 0 # 卖8量
	,	"a9_volume": int( data[66] ) if data[66] else 0 # 卖9量
	,   "a10_volume": int( data[67] ) if data[67] else 0 #卖10量
	}
	return quotation

def deal_to_dict( data ):
	deal = {
			"data_type" : 'deal'
		,   "symbol" : data[1]   # 股票代码
		,   "index"  : deal[2]     #成交序号
		,   "time" : deal[3]    # 时间，字符串格式，不带日期
		,   "price" :  float( deal[4] )  # 成交价格
		,   "volume" : int( deal[5] )  # 成交量
		,   "amount" : float( deal[6] ) # 成交金额
		,   "buynum": int( deal[7] ) # 买单委托序号
		,   "sellnum": int( deal[8] ) # 卖单委托序号
		,   "iotype": int( deal[9] ) #主动性买卖标识
		,   "channel" : int( deal[10] ) #成交通道（这是交易所的一个标记，没有作用）
	}
	return deal


def symbol_type( symbol ):
	"""
	description: 用于判断股票代码类型：母基金，分级基金，指数，AB股
	return
	------
	"""
	return symbol_type



# -*- coding: utf-8 -*-
"""
Producer类（生产者类，负责把数据推送进消息队列）
所有producer需要继承这个类
Created on 03/27/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import multiprocessing
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import time
from abc import ABCMeta
import threading
import logging

class Producer(threading.Thread):
	__metaclass__ = ABCMeta
	def __init__(self, name, **kwargs):
		super().__init__()
		self.logger = self.get_logger()
		self._name = name
		self._subscriber = set()
		self._active = False
		self._running = False

	def get_logger(self):
		logger = logging.getLogger(self.__class__.__name__)
		return logger

	def _activate(self):
		self._active = True

	def _deactivate(self):
		self._active = False
		print(self._name,self._active)

	def run(self):
		self.logger.info('[开启生产者]\t %s'% self._name)
		self._running = True
		while self._running:
			try:
				self.handler()
			except Exception as e:
				self.logger.error( "{}".format( e ) )
			time.sleep(1)
		self._end()

	def is_active(self):
		return self._active

	def is_running(self):
		return self._running

	def handler(self):
		pass

	def _end(self):
		self.logger.info("[Producer结束] %s" % self._name)

	# 添加订阅者
	def _add_subscriber(self, queue):
		self._subscriber.add(queue)
		self._activate()

	# 删除订阅者
	def _remove_subscriber(self, queue):
		self._subscriber.remove(queue)
		if ( len(self._subscriber) == 0 ):
			self.logger.info("暂停Producer %s" % self._name)
			self._deactivate()



# -*- coding: utf-8 -*-
"""
Action类
每个Action自带一个Queue
Created on 03/30/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import multiprocessing
import time
import threading
import logging
from dHydra.core.Globals import *
from dHydra.core.Functions import *
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.ThreadManager as ThreadManager
from abc import ABCMeta

class Action(threading.Thread):
	__metaclass__ = ABCMeta
	def __init__(	self
				,	name = None
				,	producer_list = list()
				,	num_start = 5
				,	num_min = 2
				,	num_max = 100
				,	need_new_thread = None
				,	cancel_thread = None
				,	on_finished = None
				,	set_daemon = True
				,	lower_threshold = 1
				,	upper_threshold = 3000	# 当消息队列数量超过upper_threshold时候，会动态添加
				,	**kwargs
				):
		super().__init__()
		self.logger = self.get_logger()
		self._name = name
		self._kwargs = kwargs
		self._queue = multiprocessing.Queue()
		self._producers = set()
		self._lock = multiprocessing.Lock()
		self._producer_list = producer_list

		self.lower_threshold = lower_threshold
		self.upper_threshold = upper_threshold

		self._manager = ThreadManager.Manager(	target = self.thread_target
											,   set_daemon = set_daemon				# 将线程设置为守护线程
											,   num_start = num_start				# 初始化action时候一次性创建的线程数
											,   num_min = num_min					# 允许最少用于执行handler的线程数
											,   num_max = num_max					# 允许用于执行handler最多线程数
											,   need_new_thread = self.need_new_thread	# 用于判断是否需要增加线程的函数，默认
											,   cancel_thread = self.cancel_thread		# 用于需要减少线程的判断函数，默认为
											,   on_finished = self.handler_callback		# handler的回调函数
											)

		# _running决定action是否开启“定时处理”
		# _active决定action是否会执行数据处理过程
		# -----------------------------------
		# 二者既不充分也不必要：
		# 	_running==False & _active==True		: "事件触发"状态
		#	_running==True & _active==True		: "定时触发"状态
		#	_running==True & _active==False		: 定时线程开启但是暂停处理数据
		#	_running==False & _active==False	: Action完全停止
		self._active = False		# _active表示：是否处理数据
		self._running = False		# _running表示：( Deprecated ) Action是否开启，这个flag无用
		self._auto_load_producers()

	def get_logger(self):
		logger = logging.getLogger(self.__class__.__name__)
		return logger

	def handler_callback( self, result ):
		return None
		# print( result )

	def need_new_thread( self ):
		"""
		需要动态新增线程的判断函数
		--------
		return:	True/False
		"""
		if self._queue.qsize() > self.upper_threshold:
			self.logger.info( "消息队列中消息数量: {}, 需要增加线程".format( self._queue.qsize() ) )
			return True
		else:
			return False

	# 需要减少线程的判断函数
	def cancel_thread( self ):
		"""
		需要动态减少线程的判断函数
		-------
		return:	True/False
		"""
		if ( self._queue.qsize() < self.lower_threshold ):
			self.logger.debug( "消息队列中消息数量: {}, 需要减少线程".format( self._queue.qsize() ) )
			return True
		else:
			return False

	# action订阅producer
	def _subscribe(self, instance):
		instance._add_subscriber(queue = self._queue)

	# action取消订阅producer
	def _unsubscribe(self, instance):
		instance._remove_subscriber(queue = self._queue)

	def _activate(self):
		self.logger.info('[激活Action]\t{}'.format( self._name ) )
		self._active = True

	def _deactivate(self):
		self._active = False
		for producer in list(self._producers):
			producer._remove_subscriber( self._queue )
		self.logger.info( '[暂停Action]\t{}'.format( self._name ) )

	# 运行action即相当于采用“定时处理”的模式
	def run(self):
		self._auto_start_producers()
		self._running = True
		self._activate()
		self.logger.info('[开启Action]\t{}'.format( self._name ) )
		self._manager.start()
		while self._running is True:
			if self._active is True:
				time.sleep(3)
		self._end()

	def is_active(self):
		return self._active

	def is_running(self):
		return self._running

	# 通知action进行一次数据处理，这是“事件触发”的模式
	def _notify(self):
		self._handler()

	def _end(self):
		for producer in list(self._producers):
			producer._remove_subscriber(self._queue)
		self.logger.info('[结束Action]\t{}'.format( self._name ) )

	def _stop(self):
		self._running = False

	# action根据self._producer_list来自动加载producers
	def _auto_load_producers(self):
		for kwargs in self._producer_list:
			instance = P(**kwargs, **self._kwargs)
			self._producers.add( instance )
			self._subscribe(instance)

	def _auto_start_producers(self):
		for producer in list(self._producers):
			if not producer.is_running():
				producer.start()

	def thread_target(self):
		try:
			event = self._queue.get(True, timeout = 3)
			return self.handler(event = event)
		except Exception as e:
			return None

	# 需要在子类中重写的数据处理方法
	def handler(self, event):
		print( "event.data: {}".format( event.data ) )
		return



# -*- coding: utf-8 -*-
"""
# Created on 04/12/2016
# @author: Emptyset
# @contact: Emptyset110@gmail
"""
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Action import Action
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置

from pymongo import MongoClient
import pymongo
import time
from datetime import datetime

# 以上是自动生成的 #
class SinaTickToMongoAction(Action):
	def __init__(self, name, **kwargs):
		# 用户自定义自动加载的_producer_list
		self._producer_list = [
			{	
				"name"	:	"SinaFreeQuote"
			,	"producer_name"	:	"SinaTickToMongo.SinaFreeQuote"		#这是在action内部给producer起的自定义名字，可随意。一般最好遵守<actionName.producerName>
			}
		]
		# 设置进程检查消息队列的间隔
		self._interval = 0.5
		super().__init__(name, **kwargs)
		print(self._name,"初始化")
		self.db = False
		while self.db == False:
			self.db = self.get_mongodb()
			time.sleep(2)

		# 这里需要初始化一下最近插入mongodb的时间戳
		sh = self.db.realtime.find_one( 
			{
				"symbol" : { '$lt' : 'sz' }
			}
		,	sort = [("_id", pymongo.DESCENDING)]
			)
		sz = self.db.realtime.find_one( 
			{
				"symbol" : { '$gt' : 'sz' }
			}
		,	sort = [("_id", pymongo.DESCENDING)]
			)
		if sh is not None:
			self.shTime = sh["time"]
			print("数据库中最近的SH ticktime:", self.shTime)
		if sz is not None:
			self.szTime = sz["time"]
			print("数据库中最近的SZ ticktime:", self.szTime)
		if (sh is None) and (sz is None):
			self.szTime = datetime(1970,1,1)
			self.shTime = datetime(1970,1,1)

	# 需要重写的方法
	def handler(self):
		while not self._queue.empty():
			event = self._queue.get(True)
			# 将event.data存储到mongodb
			l = len(event.data)
			if event.exchange == 'SZ':
				self.time = self.szTime
			elif event.exchange == 'SH':
				self.time = self.shTime
			if event.time > self.time:
				self.time = event.time
				self.db.realtime.insert_many( event.data.iloc[0:l].to_dict(orient='records') )
				print("Insert Realtime Quotes Successfully, {}, {}".format(self.time, event.exchange))
			else:
				print("消息队列中获取到一个已经存过了时间戳, {}, {}".format(self.time, event.exchange) )


	def get_mongodb(self):
		# connect to mongodb named: stock
		try:
			print("尝试连接到Mongodb")
			client = MongoClient(serverSelectionTimeoutMS=1500)
			db = client.stock
			client.server_info()
			print("已经成功连接到mongodb")
			return db
		except:
			print(">>>>>>>>>>>>>>>>>>>>>连接到mongodb失败<<<<<<<<<<<<<<<<<<<")
			print("将在2秒后重试")
			return False


# -*- coding: utf-8 -*-
"""
新浪财经
FROM ssologin.js(v1.4.18)
"""
import time
import random

CLIENT = "ssologin.js(v1.4.18)"
CROSSDOMAIN_HOST = [
	"passport.weibo.com"
,	"crosdom.weicaifu.com"
,	"passport.weibo.cn"
]

HEADERS_LOGIN = {
	"Accept" : '*/*'
,	"Accept-Encoding" : 'gzip, deflate, sdch'
,	"Accept-Language" : 'en-US,en;q=0.8'
,	"Connection" : 'keep-alive'
,	"Host" : 'login.sina.com.cn'
,	"Referer" : 'http://finance.sina.com.cn/'
,	"User-Agent" : 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
}
PARAM_L2HIST = lambda symbol,page: {
	"symbol"			:	symbol
,	"callback"			:	"jQuery17209838373235483986_" + str( int(time.time()*1000) )
,	"pageNum"			:	10000
,	"page"				:	page
,	"stime"				:	'09:25:00'
,	"etime"				:	'15:05:00'
,	"sign"				:	''
,	"num"				:	'20'
,	"_"					:	int(time.time()*1000)
}
HEADERS_L2 = lambda symbol	: {
	'Host'				:	'stock.finance.sina.com.cn'
,	'Connection'		:	'keep-alive'
,	'Accept'			:	'*/*'
,	'User-Agent'		:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
,	'Referer'			:	'http://vip.stock.finance.sina.com.cn/quotes_service/view/l2_tradedetail.php?symbol=%s' % symbol
,	'Accept-Encoding'	:	'gzip, deflate, sdch'
,	'Accept-Language'	:	'en-US,en;q=0.8'
}
HEADERS_WSKT_TOKEN = lambda 	:	{
	'Accept'					:	'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'
,	'Accept-Encoding'			:	'gzip, deflate, sdch'
,	'Accept-Language'			:	'en-US,en;q=0.8'
,	'Connection'				:	'keep-alive'
,	'Host'						:	'current.sina.com.cn'
,	'User-Agent'				:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
,	'Referer'					:	'http://finance.sina.com.cn/realstock/company/sz300204/l2.shtml'
}
HEADERS_CROSSDOMAIN = lambda host	:	{
	'Host'							:	host
,	'User-Agent'					:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
,	'Accept'						: 	'*/*'
,	'Accept-Language'				:	'en-US,en;q=0.5'
,	'Accept-Encoding'				:	'gzip, deflate'
,	'Referer'						:	'http://finance.sina.com.cn/realstock/company/sz300204/l2.shtml'
,	'Connection'					:	'keep-alive'
}
PARAM_LOGIN = lambda : {
	"client" 		:	CLIENT
,	"_"				:	int(time.time()*1000)
}
PARAM_PRELOGIN = lambda su : {
	"entry"			:	"finance"
,	"callback"		:	"sinaSSOController.preloginCallBack"
,	"su"			:	su
,	"rsakt"			:	'mod'
,	"client"		:	CLIENT
,	"_"				:	int(time.time()*1000)
}
PARAM_WSKT_TOKEN = lambda ip, qlist, hq: {
	"query"	:	hq
,	"ip"	:	ip
,	"_"		:	random.uniform(0.1,0.2)
,	"kick"	:	1
,	"list"	:	qlist
}

URL_CROSSDOMAIN = "http://login.sina.com.cn/sso/crossdomain.php"
URL_SSOLOGIN = "http://login.sina.com.cn/sso/login.php"
URL_SSOLOGOUT = "http://login.sina.com.cn/sso/logout.php"
URL_UPDATECOOKIE = "http://login.sina.com.cn/sso/updatetgt.php"
URL_PRELOGIN = "http://login.sina.com.cn/sso/prelogin.php"
URL_PINCODE = "http://login.sina.com.cn/cgi/pin.php"
URL_VFVALID = "http://weibo.com/sguide/vdun.php"

URL_L2HIST = 'http://stock.finance.sina.com.cn/stock/api/openapi.php/StockLevel2Service.getTransactionList'
URL_WSKT_TOKEN = 'https://current.sina.com.cn/auth/api/jsonp.php/var%20KKE_auth_OSfOoonMj=/AuthSign_Service.getSignCode'
SOCKET_BASE = 'ws://ff.sinajs.cn/wskt'



# -*- coding: utf-8 -*-
"""
主控制程序入口
"""
import traceback
import sys
import os

# print("加载：app.py")

from dHydra.core.Globals import *
from dHydra.core.Functions import *

def start_action(action_list, action_args = {}):
	for action in action_list:
		if action in action_args.keys():
			print(action_args[action])
			actionInstance = A(action, **action_args[action])
		else:
			actionInstance = A(action)
		action_dict[action] = actionInstance
	for action in action_list:
		action_dict[action].start()
	for action in action_list:
		action_dict[action].join()


# -*- coding: utf-8 -*-
"""
Config
Created on 03/08/2016
@description:	Connection for movie.py
@author: 		Wen Gu
@contact: 		emptyset110@gmail.com
"""

HEADERS_MAOYAN = {
	'Accept' : '*/*'
,	'Accept-Encoding'	:	'gzip, deflate, sdch'
,	'Accept-Language'	:	'en-US,en;q=0.8'
,	'Connection'		:	'keep-alive'
,	'Host'				:	'pf.maoyan.com'
,	'Referer'			:	'http://pf.maoyan.com/'
,	'User-Agent'		:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.75 Safari/537.36'
,	'X-Requested-With'	:	'XMLHttpRequest'
}

URL_MAOYAN_DATE = 'http://pf.maoyan.com/history/date/box.json'
URL_MAOYAN_MOVIE = 'http://pf.maoyan.com/history/movie/box.json'

URL_MAOYAN_BASEINFO = 'http://pf.maoyan.com/movie/baseinfo.json'
#http://pf.maoyan.com/movie/baseinfo.json?movie=246286
URL_MAOYAN_BOXINFO = 'http://pf.maoyan.com/movie/boxinfo.json'
#http://pf.maoyan.com/movie/boxinfo.json?movie=246286

URL_MAOYAN_SEATRANK = 'http://pf.maoyan.com/show/seatRank.json'

# 返回特定城市
URL_MAOYAN_SHOWRATE = "http://pf.maoyan.com/show/rate/rank.json?periodType=0&cityType=0&cityName=%E6%9D%AD%E5%B7%9E"

# 返回所有城市
URL_MAOYAN_SHOWRANK = 'http://pf.maoyan.com/show/rate/city/rank.json'

# http://pf.maoyan.com/show/rate/city/rank.json?showDate=2015-01-15&periodType=0
URL_MAOYAN_ATTENDRANK = 'http://pf.maoyan.com/attend/rate/rank.json'

URL_MAOYAN_WISH_DATE = 'http://pf.maoyan.com/movie/wish/date.json'
URL_MAOYAN_WISH_CITY = 'http://pf.maoyan.com/movie/wish/city.json'

PARAM_WISH = lambda movie: {
	"movie"		:	movie
}

PARAM_DATE = lambda date: {
	"date"	:	date
,	"cnt"	:	10
}

PARAM_MOVIEID = lambda movie: {
	"movie"		:	movie
,	"offset"	:	0
,	"limit"		:	10000
}

PARAM_SEATRANK = lambda showDate, periodType, cityType, cityName: {
	"showDate"		:	showDate
,	"periodType"	:	periodType
,	"cityType"		:	cityType
,	"cityName"		:	cityName
}


# -*- coding: utf-8 -*-
"""
# Created on 
# @author: 
# @contact: 
"""
import tornado.ioloop
import tornado.web
from dHydra.core.Globals import *

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write( "{}".format( json.dumps(action_dict,indent=2) ) )

def make_app():
    return tornado.web.Application([
        (r"/", MainHandler),
    ])

if __name__ == "__main__":
    app = make_app()
    app.listen(8888)
    tornado.ioloop.IOLoop.current().start()


# -*- coding: utf-8 -*-
"""
# Created on
# @author:
# @contact:
"""
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Action import Action
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置

# 以上是自动生成的 #
class DemoAction(Action):
	def __init__(self, name, **kwargs):
		# 用户自定义自动加载的_producer_list
		# 这一部分已经统一规定写进入口app.py
		# self._producer_list = [
		# 	}
		# 		"name"	:	"Demo"
		# 	,	"producer_name"	:	"Demo.Demo"		#这是在action内部给producer起的自定义名字，可随意。一般最好遵守<actionName.producerName>
		# 	}
		# ]
		# 设置进程检查消息队列的间隔
		self._interval = 0.5
		super().__init__(name, **kwargs)
		print(self._name,"初始化")

	# 需要重写的方法
	def handler(self):
		while not self._queue.empty():
			self._lock.acquire()
			event = self._queue.get(True)
			self._lock.release()
			print("DemoAction:{}".format( event.data ) )
